@@ -1,11 +1,12 @@
-__version__ = "2020.4.0"
+__version__ = "2021.2.1"
 __nabu_modules__ = [
     "app",
     "cuda",
-    "distributed",
+    "estimation",
     "io",
     "misc",
     "opencl",
+    "pipeline",
     "preproc",
     "reconstruction",
     "resources",
