@@ -1,18 +1,17 @@
 import os
-from typing import Union, Any
+from typing import Union
 import numpy as np
+from silx.io.dictdump import h5todict
 from tomoscan.io import HDF5File
-from .utils import get_compacted_dataslices
+from .utils import get_compacted_dataslices, convert_dict_values
 from ..misc.binning import get_binning_function
-from ..utils import subsample_dict
-
+from ..utils import subsample_dict, get_3D_subregion
 try:
     from silx.third_party.EdfFile import EdfFile
 except ImportError:
     EdfFile = None
 
-
-class Reader(object):
+class Reader:
     """
     Abstract class for various file readers.
     """
@@ -50,7 +49,7 @@ class Reader(object):
         """
         Get data from a silx.io.url.DataUrl
         """
-        raise ValueError("This should be implemented by inheriting class")
+        raise ValueError("Base class")
 
 
     def release(self):
@@ -186,17 +185,46 @@ class HDF5Reader(Reader):
         self.release()
 
 
-Readers = {
-    "edf": EDFReader,
-    "hdf5": HDF5Reader,
-    "h5": HDF5Reader,
-    "nx": HDF5Reader,
-    "npz": NPReader,
-    "npy": NPReader,
-}
+class HDF5Loader:
+    """
+    An alternative class to HDF5Reader where information is first passed at class instantiation
+    """
+    def __init__(self, fname, data_path, sub_region=None, pre_allocate=True, dtype="f"):
+        self.fname = fname
+        self.data_path = data_path
+        self._set_subregion(sub_region)
+        self.data = None
+        self._loaded = False
+        if pre_allocate:
+            expected_shape = get_hdf5_dataset_shape(fname, data_path, sub_region=sub_region)
+            self.data = np.zeros(expected_shape, dtype=dtype)
+
+    def _set_subregion(self, sub_region):
+        self.sub_region = sub_region
+        if sub_region is not None:
+            start_z, end_z, start_y, end_y, start_x, end_x = sub_region
+            self.start_x, self.end_x = start_x, end_x
+            self.start_y, self.end_y = start_y, end_y
+            self.start_z, self.end_z = start_z, end_z
+        else:
+            self.start_x, self.end_x = None, None
+            self.start_y, self.end_y = None, None
+            self.start_z, self.end_z = None, None
 
 
-class ChunkReader(object):
+    def load_data(self, force_load=False):
+        if self._loaded and not force_load:
+            return self.data
+        with HDF5File(self.fname, "r") as fdesc:
+            if self.data is None:
+                self.data = fdesc[self.data_path][self.start_z:self.end_z, self.start_y:self.end_y, self.start_x:self.end_x]
+            else:
+                self.data[:] = fdesc[self.data_path][self.start_z:self.end_z, self.start_y:self.end_y, self.start_x:self.end_x]
+        self._loaded = True
+        return self.data
+
+
+class ChunkReader:
     """
     A reader of chunk of images.
     """
@@ -259,9 +287,10 @@ class ChunkReader(object):
         the data retrieval (quite the opposite), since the whole (subregion of) data
         is read and then binning is performed.
         """
+        self._get_reader_class(files)
+        self.dataset_subsampling = dataset_subsampling
         self._set_files(files)
-        self._get_reader_class()
-        self._get_shape_and_dtype(shape, dtype, binning, dataset_subsampling)
+        self._get_shape_and_dtype(shape, dtype, binning)
         self._set_subregion(sub_region)
         self._init_reader()
         self._loaded = False
@@ -291,10 +320,11 @@ class ChunkReader(object):
         self.files = files
         self._sorted_files_indices = sorted(files.keys())
         self._fileindex_to_idx = dict.fromkeys(self._sorted_files_indices)
+        self._configure_subsampling()
 
 
-    def _infer_file_type(self):
-        fname = self.files[self._sorted_files_indices[0]].file_path()
+    def _infer_file_type(self, files):
+        fname = files[sorted(files.keys())[0]].file_path()
         ext = os.path.splitext(fname)[-1].replace(".", "")
         if ext not in Readers:
             raise ValueError(
@@ -303,29 +333,35 @@ class ChunkReader(object):
             )
         return ext
 
-    def _get_reader_class(self):
-        ext = self._infer_file_type()
+    def _get_reader_class(self, files):
+        ext = self._infer_file_type(files)
         reader_class = Readers[ext]
         self._reader_class = reader_class
 
 
-    def _get_shape_and_dtype(self, shape, dtype, binning, dataset_subsampling):
+    def _get_shape_and_dtype(self, shape, dtype, binning):
         if shape is None or dtype is None:
             shape, dtype = self._infer_shape_and_dtype()
         assert len(shape) == 2, "Expected the shape of an image (2-tuple)"
         self.shape_total = shape
         self.dtype = dtype
         self._set_binning(binning)
-        self.dataset_subsampling = dataset_subsampling
+
+
+    def _configure_subsampling(self):
+        dataset_subsampling = self.dataset_subsampling
+        self.files_subsampled = self.files
         if dataset_subsampling is not None and dataset_subsampling > 1:
             self.n_files //= dataset_subsampling
+            self.files_subsampled = subsample_dict(self.files, dataset_subsampling)
             if not(self._reader_class.multi_load):
                 # 3D loading not supported for this reader.
                 # Data is loaded frames by frame, so subsample directly self.files
-                self.files = subsample_dict(self.files, dataset_subsampling)
+                self.files = self.files_subsampled
                 self._sorted_files_indices = sorted(self.files.keys())
                 self._fileindex_to_idx = dict.fromkeys(self._sorted_files_indices)
 
+
     def _infer_shape_and_dtype(self):
         self._reader_entire_image = self._reader_class()
         first_file_dataurl = self.files[self._sorted_files_indices[0]]
@@ -408,7 +444,6 @@ class ChunkReader(object):
     def _load_multi(self):
         urls_compacted = get_compacted_dataslices(self.files, subsampling=self.dataset_subsampling)
         loaded = {}
-        data = []
         start_idx = 0
         for idx in self._sorted_files_indices:
             url = urls_compacted[idx]
@@ -443,4 +478,121 @@ class ChunkReader(object):
             self._load_single()
         self._loaded = True
 
+    load_data = load_files
 
+    @property
+    def data(self):
+        return self.files_data
+
+
+Readers = {
+    "edf": EDFReader,
+    "hdf5": HDF5Reader,
+    "h5": HDF5Reader,
+    "nx": HDF5Reader,
+    "npz": NPReader,
+    "npy": NPReader,
+}
+
+
+def load_images_from_dataurl_dict(data_url_dict, **chunk_reader_kwargs):
+    """
+    Load a dictionary of dataurl into numpy arrays.
+
+    Parameters
+    ----------
+    data_url_dict: dict
+        A dictionary where the keys are integers (the index of each image in the dataset),
+        and the values are numpy.ndarray (data_url_dict).
+
+    Other parameters
+    -----------------
+    chunk_reader_kwargs: params
+        Named parameters passed to `nabu.io.reader.ChunkReader`.
+
+    Returns
+    --------
+    res: dict
+        A dictionary where the keys are the same as `data_url_dict`, and the values are numpy arrays.
+    """
+    chunk_reader = ChunkReader(data_url_dict, **chunk_reader_kwargs)
+    img_dict = {}
+    for img_idx, img_url in data_url_dict.items():
+        img_dict[img_idx] = chunk_reader.get_data(img_url)
+    return img_dict
+
+
+def load_images_stack_from_hdf5(fname, h5_data_path, sub_region=None):
+    """
+    Load a 3D dataset from a HDF5 file.
+
+    Parameters
+    -----------
+    fname: str
+        File path
+    h5_data_path: str
+        Data path within the HDF5 file
+    sub_region: tuple, optional
+        Tuple indicating which sub-volume to load, in the form (xmin, xmax, ymin, ymax, zmin, zmax)
+        where the 3D dataset has the python shape (N_Z, N_Y, N_X).
+        This means that the data will be loaded as `data[zmin:zmax, ymin:ymax, xmin:xmax]`.
+    """
+    xmin, xmax, ymin, ymax, zmin, zmax = get_3D_subregion(sub_region)
+    with HDF5File(fname, "r") as f:
+        d_ptr = f[h5_data_path]
+        data = d_ptr[zmin:zmax, ymin:ymax, xmin:xmax]
+    return data
+
+
+def get_hdf5_dataset_shape(fname, h5_data_path, sub_region=None):
+    zmin, zmax, ymin, ymax, xmin, xmax = get_3D_subregion(sub_region)
+    with HDF5File(fname, "r") as f:
+        d_ptr = f[h5_data_path]
+        shape = d_ptr.shape
+    n_z, n_y, n_x = shape
+    # perhaps there is more elegant
+    res_shape = []
+    for n, bounds in zip([n_z, n_y, n_x], ((zmin, zmax), (ymin, ymax), (xmin, xmax))):
+        res_shape.append(np.arange(n)[bounds[0]:bounds[1]].size)
+    return tuple(res_shape)
+
+
+def check_virtual_sources_exist(fname, data_path):
+    with HDF5File(fname, "r") as f:
+        if data_path not in f:
+            print("No dataset %s in file %s" % (data_path, fname))
+            return False
+        dptr = f[data_path]
+        if not dptr.is_virtual:
+            return True
+        for vsource in dptr.virtual_sources():
+            vsource_fname = os.path.join(os.path.dirname(dptr.file.filename), vsource.file_name)
+            if not os.path.isfile(vsource_fname):
+                print("No such file: %s" % vsource_fname)
+                return False
+            elif not check_virtual_sources_exist(vsource_fname, vsource.dset_name):
+                print("Error with virtual source %s" % vsource_fname)
+                return False
+    return True
+
+
+def import_h5_to_dict(h5file, h5path, asarray=False):
+    """
+    Wrapper on top of silx.io.dictdump.dicttoh5 replacing "None" with None
+
+    Parameters
+    -----------
+    h5file: str
+        File name
+    h5path: str
+        Path in the HDF5 file
+    asarray: bool, optional
+        Whether to convert each numeric value to an 0D array. Default is False.
+    """
+    dic = h5todict(h5file, path=h5path, asarray=asarray)
+    modified_dic = convert_dict_values(
+        dic,
+        {"None": None},
+        bytes_tostring=True
+    )
+    return modified_dic
