@@ -9,7 +9,7 @@ if __has_pycuda__:
     import pycuda.gpuarray as garray
     import pycuda.cumath as cumath
 
-class CudaDoubleFlatField(DoubleFlatField, CudaProcessing):
+class CudaDoubleFlatField(DoubleFlatField):
 
     def __init__(
         self,
@@ -21,13 +21,13 @@ class CudaDoubleFlatField(DoubleFlatField, CudaProcessing):
         average_is_on_log=False,
         sigma_filter=None,
         filter_mode="reflect",
-        cuda_options={},
+        cuda_options=None,
     ):
         """
         Init double flat field with Cuda backend.
         """
-        DoubleFlatField.__init__(
-            self,
+        self.cuda_processing = CudaProcessing(**(cuda_options or {}))
+        super().__init__(
             shape,
             result_url=result_url,
             sub_region=sub_region,
@@ -37,7 +37,6 @@ class CudaDoubleFlatField(DoubleFlatField, CudaProcessing):
             sigma_filter=sigma_filter,
             filter_mode=filter_mode
         )
-        CudaProcessing.__init__(self, **cuda_options)
         self._init_gaussian_filter()
 
 
@@ -192,5 +191,3 @@ class CudaDoubleFlatField(DoubleFlatField, CudaProcessing):
         for i in range(self.n_angles):
             radios[i] /= dff
         return radios
-
-
