@@ -1,11 +1,12 @@
-import numpy as np
-import os
+from collections import namedtuple
 from itertools import product
+import os
+import numpy as np
 from silx.resources import ExternalResources
 from silx.io.dictdump import dicttoh5
+from silx.io.url import DataUrl
 from tomoscan.io import HDF5File
-from .resources.validators import convert_to_bool
-
+from .io.utils import get_compacted_dataslices
 
 utilstest = ExternalResources(
     project="nabu",
@@ -20,7 +21,10 @@ if __big_testdata_dir__ is None or not(os.path.isdir(__big_testdata_dir__)):
 
 __do_long_tests__ = os.environ.get("NABU_LONG_TESTS", False)
 if __do_long_tests__:
-    __do_long_tests__ = convert_to_bool(__do_long_tests__)
+    try:
+        __do_long_tests__ = bool(int(__do_long_tests__))
+    except:
+        __do_long_tests__ = False
 
 
 def generate_tests_scenarios(configurations):
@@ -120,6 +124,25 @@ def compare_arrays(arr1, arr2, tol, diff=None, absolute_value=True, percent=None
     return res
 
 
+class SimpleHDF5TomoScanMock:
+    def __init__(self, image_key):
+        self._image_key = image_key
+
+    @property
+    def image_key(self):
+        return self._image_key
+
+    @image_key.setter
+    def image_key(self, image_key):
+        self._image_key = image_key
+
+    def save_reduced_flats(self, *args, **kwargs):
+        pass
+
+    def save_reduced_darks(self, *args, **kwargs):
+        pass
+
+
 class NXDatasetMock:
     """
     An alternative to tomoscan.esrf.mock.MockHDF5, with a different interface.
@@ -139,6 +162,12 @@ class NXDatasetMock:
         assert image_keys.size == self.n_proj
         self._finalize_init(other_params)
         self.dataset_dict = None
+        self.fname = None
+        # Mocks more attributes
+        self.dataset_scanner = SimpleHDF5TomoScanMock(
+            image_key=self.image_key
+        )
+        self.kind = "hdf5"
 
 
     def _finalize_init(self, other_params):
@@ -201,6 +230,7 @@ class NXDatasetMock:
 
 
     def generate_hdf5_file(self, fname, h5path=None):
+        self.fname = fname
         h5path = h5path or "/entry"
         if self.dataset_dict is None:
             self.generate_dict()
@@ -208,8 +238,49 @@ class NXDatasetMock:
             self.dataset_dict, fname,  h5path=h5path, mode="a"
         )
         # Patch the "data" field which is exported as string by dicttoh5 (?!)
-        dataset_path = os.path.join(h5path, "instrument/detector/data")
+        self.dataset_path = os.path.join(h5path, "instrument/detector/data")
         with HDF5File(fname, "a") as fid:
-            del fid[dataset_path]
-            fid[dataset_path] = self.dataset_dict["instrument"]["detector"]["data"]
+            del fid[self.dataset_path]
+            fid[self.dataset_path] = self.dataset_dict["instrument"]["detector"]["data"]
 
+
+    # Mock some of the HDF5DatasetAnalyzer attributes
+    @property
+    def dataset_hdf5_url(self):
+        if self.fname is None:
+            raise ValueError("generate_hdf5_file() was not called")
+        return DataUrl(
+            file_path=self.fname, data_path=self.dataset_path, scheme="silx"
+        )
+
+    def _get_images_with_key(self, key):
+        indices = np.arange(self.image_key.size)[self.image_key == key]
+        urls = [
+            DataUrl(
+                file_path=self.fname,
+                data_path=self.dataset_path,
+                data_slice=slice(img_idx, img_idx+1),
+                scheme="silx",
+            )
+            for img_idx in indices
+        ]
+        return dict(zip(indices, urls))
+
+    @property
+    def flats(self):
+        return self._get_images_with_key(1)
+
+    @property
+    def darks(self):
+        return self._get_images_with_key(2)
+
+
+    def get_data_slices(self, what):
+        images = getattr(self, what)
+        # we can't directly use set() on slice() object (unhashable). Use tuples
+        tuples_list = list(set(
+            (du.data_slice().start, du.data_slice().stop)
+            for du in get_compacted_dataslices(images).values()
+        ))
+        slices_list = [slice(item[0], item[1]) for item in tuples_list]
+        return slices_list
